<?php

/**
 * ownCloud
 *
 * @author Dominik Schmidt
 * @copyright 2011 Dominik Schmidt dev@dominik-schmidt.de
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU AFFERO GENERAL PUBLIC LICENSE
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU AFFERO GENERAL PUBLIC LICENSE for more details.
 *
 * You should have received a copy of the GNU Affero General Public
 * License along with this library.  If not, see <http://www.gnu.org/licenses/>.
 *
 */
class YG_INTERNAL_LDAP_WRAPPER {

    static protected $ds;
    static protected $configured = false;
    // cached settings
    static protected $ldap_host;
    static protected $ldap_port;
    static protected $ldap_dn;
    static protected $ldap_password;
    static protected $ldap_base;
    static protected $ldap_tls;
    static protected $ldap_nocase;

    static protected function init() {
        if (!self::$configured) {
            self::$ldap_host = OC_Appconfig::getValue('global_ldap', 'ldap_host', '');
            self::$ldap_port = OC_Appconfig::getValue('global_ldap', 'ldap_port', OC_USER_BACKEND_GLDAP_DEFAULT_PORT);
            self::$ldap_dn = OC_Appconfig::getValue('global_ldap', 'ldap_dn', '');
            self::$ldap_password = OC_Appconfig::getValue('global_ldap', 'ldap_password', '');
            self::$ldap_base = OC_Appconfig::getValue('global_ldap', 'ldap_base', '');
            self::$ldap_tls = OC_Appconfig::getValue('global_ldap', 'ldap_tls', 0);
            self::$ldap_nocase = OC_Appconfig::getValue('global_ldap', 'ldap_nocase', 0);
            self::$configured = true;
        }
    }

    static public function ldap_filterWith($searchAttribute, $searchValue, $userFilter) {

        $attributeFilter = '(' . $searchAttribute . '=' . $searchValue . ')';

        if (!empty($userFilter)) {
            return '(&' . $attributeFilter . $userFilter . ')';
        }

        return $attributeFilter;
    }

    /**
     * @brief Check if the password is correct
     * @param $dn The user dn
     * @param $password The password
     * @returns true/false
     *
     * Check if the password is correct without logging in the user
     */
    static public function ldap_checkAuth($dn, $password) {
        self::init();
        $ldap_c = ldap_connect(self::$ldap_host, self::$ldap_port);
        if (ldap_set_option($ldap_c, LDAP_OPT_PROTOCOL_VERSION, 3))
            if (ldap_set_option($ldap_c, LDAP_OPT_REFERRALS, 0))
                if (self::$ldap_tls)
                    if (!ldap_start_tls($ldap_c))
                        return false;


        $success = @ldap_bind($ldap_c, $dn, $password);

        return $success;
    }

    static protected function ldap_get_service_session() {
        self::init();
        if (!self::$ds) {
            self::$ds = ldap_connect(self::$ldap_host, self::$ldap_port);
            if (ldap_set_option(self::$ds, LDAP_OPT_PROTOCOL_VERSION, 3))
                if (ldap_set_option(self::$ds, LDAP_OPT_REFERRALS, 0))
                    if (self::$ldap_tls)
                        if (!ldap_start_tls(self::$ds))
                            return false;
        }
        // login
        if (!empty(self::$ldap_dn)) {
            $ldap_login = @ldap_bind(self::$ds, self::$ldap_dn, self::$ldap_password);
            if (!$ldap_login) {
                //TODO: Best error handling
                return false;
            }
        }

        return self::$ds;
    }

    static public function ldap_get_entries_with_attrs($filter, $attrs) {
        // connect to server
        self::$ds = self::ldap_get_service_session();
        if (!self::$ds)
            return array();

        $sr = ldap_search(self::$ds, self::$ldap_base, $filter, $attrs);
        $entries = ldap_get_entries(self::$ds, $sr);

        return $entries;
    }

    static public function ldap_get_entries_with_attr($filter, $attr) {
        $entries = self::ldap_get_entries_with_attrs($filter, array($attr));
        if (count($entries) == 0)
            return array();
        else {
            $values = array();
            foreach ($entries as $row) {
                if (isset($row[strtolower($attr)])) {
                    $values = array_merge($values, $row[strtolower($attr)]);
                }
            }
            unset($values['count']);
            return array_unique($values, SORT_LOCALE_STRING);
        }
    }

    static public function ldap_get_entries_with_first_attr($filter, $attr) {
        $entries = self::ldap_get_entries_with_attrs($filter, array($attr));
        if (count($entries) == 0)
            return array();
        else {
            $values = array();
            foreach ($entries as $row) {
                if (isset($row[strtolower($attr)])) {
                    $values[] = $row[strtolower($attr)][0];
                }
            }
            return array_unique($values, SORT_LOCALE_STRING);
        }
    }

}

class OC_USER_GLDAP extends OC_User_Backend {

    protected $ds;
    protected $configured = false;
    // cached settings
    protected $ldap_host;
    protected $ldap_port;
    protected $ldap_dn;
    protected $ldap_password;
    protected $ldap_base;
    protected $ldap_login_attr;  // rules to match user at login
    protected $ldap_user_filter; // rules to list users allowed to be displayed
    protected $ldap_tls;
    protected $ldap_nocase;
    protected $ldap_quota;
    protected $ldap_quota_def;
    protected $ldap_email;
    // will be retrieved from LDAP server
    protected $ldap_user_infos = array();

    function __construct() {
        $this->ldap_host = OC_Appconfig::getValue('global_ldap', 'ldap_host', '');
        $this->ldap_port = OC_Appconfig::getValue('global_ldap', 'ldap_port', OC_USER_BACKEND_GLDAP_DEFAULT_PORT);
        $this->ldap_dn = OC_Appconfig::getValue('global_ldap', 'ldap_dn', '');
        $this->ldap_password = OC_Appconfig::getValue('global_ldap', 'ldap_password', '');
        $this->ldap_base = OC_Appconfig::getValue('global_ldap', 'ldap_base', '');
        $this->ldap_login_attr = OC_Appconfig::getValue('global_ldap', 'ldap_login_attr', OC_USER_BACKEND_GLDAP_DEFAULT_LOGIN_ATTRIBUTE);
        $this->ldap_user_filter = OC_Appconfig::getValue('global_ldap', 'ldap_user_filter', '');
        $this->ldap_tls = OC_Appconfig::getValue('global_ldap', 'ldap_tls', 0);
        $this->ldap_nocase = OC_Appconfig::getValue('global_ldap', 'ldap_nocase', 0);
        $this->ldap_quota_attr = OC_Appconfig::getValue('global_ldap', 'ldap_quota_attr', '');
        $this->ldap_quota_def = OC_Appconfig::getValue('global_ldap', 'ldap_quota_def', '');
        $this->ldap_email_attr = OC_Appconfig::getValue('global_ldap', 'ldap_email_attr', '');


        if (!empty($this->ldap_host)
                && !empty($this->ldap_port)
                && ((!empty($this->ldap_dn) && !empty($this->ldap_password)) || (empty($this->ldap_dn) && empty($this->ldap_password)))
                && !empty($this->ldap_base)
                && !empty($this->ldap_login_attr)
        ) {
            $this->configured = true;
        }
    }

    function __destruct() {
        
    }

    private function setQuota($uid) {
        if (!empty($this->ldap_quota_attr)) {
            $quota = $this->ldap_user_infos[strtolower($this->ldap_quota_attr)][0];
        } else {
            $quota = false;
        }
        $quota = $quota != -1 ? $quota : $this->ldap_quota_def;
        OC_Preferences::setValue($uid, 'files', 'quota', OC_Helper::computerFileSize($quota));
    }

    private function setEmail($uid) {
        $email = $this->ldap_user_infos[$this->ldap_email_attr][0];
        OC_Preferences::setValue($uid, 'settings', 'email', $email);
    }

    /**
     * @brief Get LDAP informations for a user
     * @param $uid The username
     * @returns a array representing the user
     *
     * Return the first entry with the specified username
     */
    public function getUserInfo($uid) {
        if (!$this->configured)
            return false;

        $filter = YG_INTERNAL_LDAP_WRAPPER::ldap_filterWith($this->ldap_login_attr, $uid, $this->ldap_user_filter);
        $entries = YG_INTERNAL_LDAP_WRAPPER::ldap_get_entries_with_attrs($filter, array($this->ldap_login_attr));

        if (count($entries) == 0) {
            return array();
        }

        $this->ldap_user_infos = $entries[0];

        return $this->ldap_user_infos;
    }

    /**
     * @brief Check if the password is correct
     * @param $uid The username
     * @param $password The password
     * @returns the user id or false
     *
     * Check if the password is correct without logging in the user
     */
    public function checkPassword($uid, $password) {

        if (!$this->configured) {
            return false;
        }

        $user = $this->getUserInfo($uid);

        if (count($user) == 0) {
            return false;
        }

        $boolResult = YG_INTERNAL_LDAP_WRAPPER::ldap_checkAuth($user['dn'], $password);

        return $boolResult ? $user[strtolower($this->ldap_login_attr)][0] : false;
    }

    /**
     * @brief check if a user exists
     * @param string $uid the username
     * @return boolean
     */
    public function userExists($uid) {
        if (!$this->configured) {
            return false;
        }
        $dn = $this->getUserInfo($uid);
        return !empty($dn);
    }

    /**
     * @brief Get a list of all users
     * @returns array with all uids
     *
     * Get a list of all users.
     */
    public function getUsers() {
        if (!$this->configured)
            return array();

        $filter = YG_INTERNAL_LDAP_WRAPPER::ldap_filterWith($this->ldap_login_attr, '*', $this->ldap_user_filter);
        $entries = YG_INTERNAL_LDAP_WRAPPER::ldap_get_entries_with_first_attr($filter, $this->ldap_login_attr);

        return $entries;
    }

}

class OC_GROUP_GLDAP { // extends OC_Group_Backend {

    protected $ds;
    protected $configured = false;
    // cached settings
    protected $ldap_host;
    protected $ldap_port;
    protected $ldap_dn;
    protected $ldap_password;
    protected $ldap_base;
    protected $ldap_login_attr;  // rules to match user at login
    protected $ldap_user_filter; // rules to list users allowed to be displayed
    protected $ldap_tls;
    protected $ldap_nocase;
    protected $ldap_group_enabled;
    protected $ldap_group_display_attr;
    protected $ldap_group_member_attr;
    protected $ldap_group_filter;
    protected $ldap_group_admin;
    // will be retrieved from LDAP server
    protected $ldap_dc = false;

    function __construct() {
        $this->ldap_host = OC_Appconfig::getValue('global_ldap', 'ldap_host', '');
        $this->ldap_port = OC_Appconfig::getValue('global_ldap', 'ldap_port', OC_USER_BACKEND_GLDAP_DEFAULT_PORT);
        $this->ldap_dn = OC_Appconfig::getValue('global_ldap', 'ldap_dn', '');
        $this->ldap_password = OC_Appconfig::getValue('global_ldap', 'ldap_password', '');
        $this->ldap_base = OC_Appconfig::getValue('global_ldap', 'ldap_base', '');
        $this->ldap_login_attr = OC_Appconfig::getValue('global_ldap', 'ldap_login_attr', '');
        $this->ldap_user_filter = OC_Appconfig::getValue('global_ldap', 'ldap_user_filter', 'objectClass=person');
        $this->ldap_tls = OC_Appconfig::getValue('global_ldap', 'ldap_tls', 0);
        $this->ldap_nocase = OC_Appconfig::getValue('global_ldap', 'ldap_nocase', 0);

        $this->ldap_group_display_attr = OC_Appconfig::getValue('global_ldap', 'ldap_group_display_attr', 'cn');
        $this->ldap_group_member_attr = OC_Appconfig::getValue('global_ldap', 'ldap_group_member_attr', 'memberuid');
        $this->ldap_group_member_attr_use_dn = OC_Appconfig::getValue('global_ldap', 'ldap_group_member_attr_use_dn', false);
        $this->ldap_group_filter = OC_Appconfig::getValue('global_ldap', 'ldap_group_filter', 'objectClass=posixgroup');
        $this->ldap_group_admin = OC_Appconfig::getValue('global_ldap', 'ldap_group_admin', 'admin');

        if (!empty($this->ldap_host)
                && !empty($this->ldap_port)
                && ((!empty($this->ldap_dn) && !empty($this->ldap_password)) || (empty($this->ldap_dn) && empty($this->ldap_password)))
                && !empty($this->ldap_base)
                && !empty($this->ldap_login_attr)
        ) {
            $this->configured = true;
        }
    }

    function __destruct() {
        
    }

    /**
     * @brief get a list of all groups
     * @returns array with group names
     *
     * Returns a list with all groups
     */
    public function getGroups() {
        if (!$this->configured)
            return array();

        $entries = YG_INTERNAL_LDAP_WRAPPER::ldap_get_entries_with_attr($this->ldap_group_filter, $this->ldap_group_display_attr);

        return $entries;
    }

    /**
     * @brief Get all groups a user belongs to
     * @param $uid Name of the user
     * @returns array with group names
     *
     * This function fetches all groups a user belongs to. It does not check
     * if the user exists at all.
     */
    public function getUserGroups($uid) {
        if (!$this->configured)
            return array();

        if ($this->ldap_group_member_attr_use_dn) {
            $userbackend = new OC_USER_GLDAP();
            $user = $userbackend->getUserInfo($uid);
            $filter = YG_INTERNAL_LDAP_WRAPPER::ldap_filterWith($this->ldap_group_member_attr, $user['dn'], $this->ldap_group_filter);
        } else {
            $filter = YG_INTERNAL_LDAP_WRAPPER::ldap_filterWith($this->ldap_group_member_attr, $uid, $this->ldap_group_filter);
        }

        $entries = YG_INTERNAL_LDAP_WRAPPER::ldap_get_entries_with_attr($filter, $this->ldap_group_display_attr);

        return $entries;
    }

    /**
     * @brief get a list of all users in a group
     * @returns array with user ids
     */
    public function usersInGroup($gid) {
        if (!$this->configured)
            return array();

        $filter = YG_INTERNAL_LDAP_WRAPPER::ldap_filterWith($this->ldap_group_display_attr, $gid, $this->ldap_group_filter);
        $entries = YG_INTERNAL_LDAP_WRAPPER::ldap_get_entries_with_attr($filter, $this->ldap_group_member_attr);

        return $entries;
    }

    /**
     * @brief is user in group?
     * @param $uid uid of the user
     * @param $gid gid of the group
     * @returns true/false
     *
     * Checks whether the user is member of a group or not.
     */
    public function inGroup($uid, $gid) {
        // Hack: ownCloud use hardcoded name for the administrator group. If get a request about the admin group, we convert it in the good LDAP admin group
        if ($gid === "admin") {
            $gid = $this->ldap_group_admin;
        }

        if ($this->ldap_group_member_attr_use_dn) {
            $userbackend = new OC_USER_GLDAP();
            $user = $userbackend->getUserInfo($uid);
            $filter = YG_INTERNAL_LDAP_WRAPPER::ldap_filterWith($this->ldap_group_display_attr, $gid, '(&' . $this->ldap_group_filter . '(' . $this->ldap_group_member_attr . '=' . $user['dn'] . '))');
        } else {
            $filter = YG_INTERNAL_LDAP_WRAPPER::ldap_filterWith($this->ldap_group_display_attr, $gid, '(&' . $this->ldap_group_filter . '(' . $this->ldap_group_member_attr . '=' . $uid . '))');
        }

        $entries = YG_INTERNAL_LDAP_WRAPPER::ldap_get_entries_with_attr($filter, $this->ldap_group_display_attr);

        return count($entries) == 0 ? false : true;
    }

    public function groupExists($gid) {
        $filter = YG_INTERNAL_LDAP_WRAPPER::ldap_filterWith($this->ldap_group_member_attr, $gid, $this->ldap_group_filter);
        $entries = YG_INTERNAL_LDAP_WRAPPER::ldap_get_entries_with_attr($filter, $this->ldap_group_display_attr);

        return count($entries) == 0 ? false : true;
    }

    // not implemented 
    public function removeGroup($gid) {
        return false;
    }

    public function createGroup($gid) {
        return false;
    }

    public function deleteGroup($gid) {
        return false;
    }

    public function addToGroup($uid, $gid) {
        return false;
    }

    public function removeFromGroup($uid, $gid) {
        return false;
    }

}

?>
